<?php

use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreateMenuTables extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('menus', function (Blueprint $table) {
            $table->increments('id');
            $table->string('name', 50);
            $table->string('description', 255);

            $table->unique('name', 'name_unique');
        });

        Schema::create('menus_items', function (Blueprint $table) {
            $table->increments('id');
            $table->integer('menu_id')->unsigned();
            $table->integer('parent_id')->unsigned();

            $locales = ['en'];
            foreach ($locales as $locale) {
                $table->string('name_' . $locale, 255);
            }
            $table->string('url', 255);
            $table->boolean('target')->default(false);
            $table->integer('sort')->unsigned();

            $table->index('sort', 'index_sort');

            $table->foreign('menu_id', 'fk_menu_items_menus')
                ->references('id')
                ->on('menus')
                ->onUpdate('cascade')
                ->onDelete('cascade');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('menus_items');

        Schema::dropIfExists('menus');
    }
}
