<?php

namespace AltSolution\Admin\Console\Commands;

use Carbon\Carbon;
use Cron\CronExpression;
use Illuminate\Console\Command;
use Illuminate\Console\Scheduling\Event;
use Illuminate\Console\Scheduling\Schedule;

class ListSchedule extends Command
{
    /**
     * @var string
     */
    protected $signature = 'schedule:list';
    /**
     * @var string
     */
    protected $description = 'List scheduled commands';
    /**
     * @var Schedule
     */
    private $schedule;

    /**
     * @param Schedule $schedule
     * @return void
     */
    public function __construct(Schedule $schedule)
    {
        parent::__construct();

        $this->schedule = $schedule;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        if (count($this->schedule->events()) > 0) {
            $events = collect($this->schedule->events())->map(function (Event $event) {
                return [
                    'schedule' => $event->expression,
                    'command' => $event->command,
                    'description' => $event->description ?: 'N/A',
                    'upcoming' => $this->upcoming($event),
                    'timezone' => $event->timezone ?: config('app.timezone'),
                    'overlaps' => $event->withoutOverlapping ? 'No' : 'Yes',
                    'maintenance' => $event->evenInMaintenanceMode ? 'Yes' : 'No',
                    //'one_server' => $event->onOneServer ? 'Yes' : 'No',
                ];
            });

            $this->table(['Schedule', 'Command', 'Description', 'Upcoming', 'Timezone', 'Overlaps?', 'In Maintenance?'/*, 'One Server?'*/], $events);
        } else {
            $this->info('No Scheduled Commands Found');
        }
    }

    /**
     * Get Upcoming schedule.
     *
     * @return bool
     */
    protected function upcoming(Event $event)
    {
        $date = Carbon::now();

        if ($event->timezone) {
            $date->setTimezone($event->timezone);
        }

        return (CronExpression::factory($event->expression)->getNextRunDate($date->toDateTimeString()))->format('Y-m-d H:i:s');
    }
}
