<?php

namespace AltSolution\Admin\Form\Field;

use AltSolution\Admin\Form\AbstractField;
use DateTimeInterface;
use Illuminate\Support\Arr;

class DateRange extends AbstractField
{
    protected $view = 'daterange';
    protected $format = 'Y-m-d';

    protected function getRules()
    {
        return [
            'date_format:' . $this->getFormat(),
        ];
    }

    protected function getFormat()
    {
        return $this->getOption('format', $this->format);
    }

    public function getValidationName()
    {
        $name = $this->getName();
        $name .= '.*';
        return $name;
    }

    public function normalizeValue($value)
    {
        $format = $this->getFormat();
        $value = (array)$value;

        $normalized = [];
        foreach (range(0,1) as $index) {
            $subValue = Arr::get($value, $index);
            if (empty($subValue)) {
                $normalized[$index] = null;
            } elseif ($subValue instanceof DateTimeInterface) {
                // we think value is DateTime or Carbon instance
                $normalized[$index ] = $subValue->format($format);
            }
            elseif (is_numeric($subValue)) {
                // we think value is timestamp
                $normalized[$index] = date($format, $subValue);
            }
            else {
                // we think wtf, maybe string
                $normalized[$index] = date($format, strtotime($subValue));
            }
        }

        return $normalized;
    }
}