<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Image\ImagesInterface;
use AltSolution\Admin\Seo\SeoInterface;
use AltSolution\Admin\Models\Content;
use AltSolution\Admin\Modules\Content\ContentFormInterface;
use AltSolution\Admin\Modules\Content\ContentModelInterface;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;

class ContentController extends Controller
{
    public function getIndex(Request $request)
    {
        $this->authorize('permission', 'content.list');

        $filterForm = cms_construct_filter();
        $filterForm->setDataSource($request);
        $filter = $filterForm->extractData();

        /** @var Builder $qb */
        $qb = app(ContentModelInterface::class)->query();
        if (!empty($filter['q'])) {
            $qb->where(function ($query) use ($filter) {
                foreach (cms_locales() as $locale) {
                    $query->orWhere('title_' . $locale, 'LIKE', '%' . $filter['q'] . '%');
                    $query->orWhere('content_' . $locale, 'LIKE', '%' . $filter['q'] . '%');
                }
            });
        }
        // TODO: refactor
        if (empty($filter['sort'])) {
            $filter['sort'] = 'abc-asc';
        }
        list($sortBy, $sortDir) = explode('-', $filter['sort']);
        if ($sortBy == 'abc') {
            $locale = cms_admin_locale();
            $qb->orderBy('title_' . $locale, $sortDir);
        } else {
            $qb->orderBy('id', $sortDir);
        }

        $items = $qb->paginate(config('admin.items_per_page', 20));
        $items->appends($filter);

        $this->layout
            ->setMenuSection('content')
            ->setTitle(trans('admin::content.title'));
        return view('admin::content.list', compact('items', 'filterForm'));
    }

    public function getEdit($id = null)
    {
        $this->authorize('permission', 'content.edit');

        $content = app(ContentModelInterface::class)->query()->findOrNew($id);
        if (!$id) {
            $content->is_published = 1;
        }

        $form = cms_create_form(ContentFormInterface::class, $content);

        $this->layout
            ->setMenuSection('content')
            ->setTitle(trans($id ? 'admin::content.edit' : 'admin::content.add'))
            ->addBreadcrumb(trans('admin::content.title'), route('admin::content_list'));
        return view('admin::content.edit', compact('form', 'content'));
    }

    public function postSave(Request $request)
    {
        $this->authorize('permission', 'content.edit');

        $itemId = $request->input('id');
        $content = app(ContentModelInterface::class)->query()->firstOrNew(['id' => $itemId]);
        $form = cms_create_form(ContentFormInterface::class, $content);
        $validData = $form->validate($request);

        /** @var Content $content */
        $content->fill($validData);
        if ($content instanceof ImagesInterface) {
            $content->imageAllSave($request);
        }
        if (!$content->exists) {
            //
        }
        $content->user_id = auth()->user()->id;
        event('admin.model.content.saving', $content);
        $content->save();
        if ($content instanceof SeoInterface) {
            $content->seoSave($request->seo);
        }
        event('admin.model.content.saved', $content);

        $this->layout->addNotify('success', trans('admin::content.saved'));

        if ($request->button_apply) {
            return redirect()->route('admin::content_edit', $content->id);
        }

        return redirect()->route('admin::content_list');
    }

    public function action(Request $request)
    {
        $action = $request->input('action');
        $ids = $request->input('ids');
        if (!$ids) {
            return;
        }

        /** @var Content $item */
        if ($action == 'publish') {
            $this->authorize('permission', 'content.edit');

            foreach ($ids as $id) {
                $item = app(ContentModelInterface::class)->query()->find($id);
                if (!$item) {
                    continue;
                }
                $item->is_published = 1;
                event('admin.model.content.saving', $item);
                $item->save();
                event('admin.model.content.saved', $item);
            }
        } elseif ($action == 'hide') {
            $this->authorize('permission', 'content.edit');

            foreach ($ids as $id) {
                $item = app(ContentModelInterface::class)->query()->find($id);
                if (!$item) {
                    continue;
                }
                $item->is_published = 0;
                event('admin.model.content.saving', $item);
                $item->save();
                event('admin.model.content.saved', $item);
            }
        } elseif ($action == 'delete') {
            $this->authorize('permission', 'content.delete');

            foreach ($ids as $id) {
                $item = app(ContentModelInterface::class)->query()->find($id);
                if (!$item) {
                    continue;
                }
                if ($item instanceof ImagesInterface) {
                    $item->imageAllDelete();
                }
                event('admin.model.content.deleting', $item);
                $item->delete();
                if ($item instanceof SeoInterface) {
                    $item->seoDelete();
                }
                event('admin.model.content.deleted', $item);
            }
        }
    }
}
