<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Forms\RoleForm;
use AltSolution\Admin\Models\AclRole;
use Illuminate\Http\Request;

class RoleController extends Controller
{
    public function getRoles()
    {
        $this->authorize('permission', 'role.list');

        $items = AclRole::query()
            ->paginate(config('admin.items_per_page', 20));
        $user = auth()->user();

        $this->layout
            ->setMenuSection('users')
            ->setTitle(trans('admin::user.role_title'));
        return view('admin::users.roles', compact('items', 'user'));
    }

    public function getEdit($id = null)
    {
        $this->authorize('permission', 'role.edit');

        $role = AclRole::query()->findOrNew($id);
        $form = cms_create_form(RoleForm::class, $role);

        $this->layout
            ->setMenuSection('users')
            ->setTitle(trans($id ? 'admin::user.role_edit' : 'admin::user.role_add'))
            ->addBreadcrumb(trans('admin::user.role_title'), route('admin::user_roles'));
        return view('admin::users.role_edit', compact('role', 'form'));
    }

    public function postRoles(Request $request)
    {
        $this->authorize('permission', 'role.edit');

        $roleId = $request->input('id');
        $role = AclRole::query()->firstOrNew(['id' => $roleId]);
        $form = cms_create_form(RoleForm::class, $role);
        $validData = $form->validate($request);

        $role->fill($validData);
        event('admin.model.role.saving', $role);
        $role->save();
        event('admin.model.role.saved', $role);

        $this->layout->addNotify('success', trans('admin::user.role_saved'));

        if ($request->button_apply) {
            return redirect()->route('admin::user_editrole', ['id' => $role->id]);
        }

        return redirect()->route('admin::user_roles');
    }

    public function action(Request $request)
    {
        $action = $request->input('action');
        $ids = $request->input('ids');
        if (!$ids) {
            return;
        }

        if ($action == 'delete') {
            $this->authorize('permission', 'role.delete');

            $currentUser = auth()->user();
            foreach ($ids as $id) {
                /** @var AclRole $role */
                $role = AclRole::query()->find($id);
                if (!$role && $role->id == $currentUser->acl_role_id) {
                    continue;
                }

                event('admin.model.role.deleting', $role);
                $role->delete();
                event('admin.model.role.deleted', $role);
            }
        }
    }
}
