<?php

namespace AltSolution\Admin\Image;

use Illuminate\Http\Request;
use Illuminate\Http\UploadedFile;

interface ImagesInterface
{
    /**
     * Save image in storage
     * @param string $property Image field
     * @param UploadedFile $file
     * @throws \Exception
     */
    function imageSave($property, UploadedFile $file);

    /**
     * Save image in storage from file
     * @param string $property Image field
     * @param \SplFileInfo $file
     * @throws \Exception
     */
    function imageSaveFile($property, \SplFileInfo $file);

    /**
     * Save image in storage from url
     * @param string $property Image field
     * @param string $url
     * @throws \Exception
     */
    function imageSaveUrl($property, $url);

    /**
     * Automatic save and delete images from storage
     * @param Request $request
     */
    function imageAllSave(Request $request);

    /**
     * Get image url for specified size
     * @param string $property Image field
     * @param string $size Image size
     * @return null|string
     */
    function imageUrl($property, $size = null);

    /**
     * Get WebP image url for specified size
     * @param string $property Image field
     * @param null|string $size Image Size
     * @return null|string
     */
    function imageWebpUrl($property, $size = null);

    /**
     * Delete image from storage
     * @param string $property Image field
     */
    function imageDelete($property);

    /**
     * Delete all images from storage
     */
    function imageAllDelete();

    /**
     * Delete image slicing from storage
     * @param string $property Image field
     */
    function imageReset($property);

    /**
     * Delete all images slicing from storage
     */
    function imageAllReset();
}

