<?php

namespace AltSolution\Admin\Listeners;

use AltSolution\Admin\Models\User;
use Illuminate\Auth\Events\Failed;
use Illuminate\Auth\Events\Lockout;
use Illuminate\Auth\Events\Login;

class AuthLogSubscriber
{
    /**
     * Register the listeners for the subscriber.
     *
     * @param  \Illuminate\Events\Dispatcher $events
     */
    public function subscribe($events)
    {
        $events->listen(Login::class, static::class . '@onLogin');
        $events->listen(Failed::class, static::class . '@onFailed');
        $events->listen(Lockout::class, static::class . '@onLockout');
        User::saved(static::class . '@onPasswordChanged');
    }

    public function onLogin(Login $event)
    {
        /** @var User $user */
        $user = $event->user;

        if ($user && ($user instanceof User)) {
            $replace = [
                'name' => $user->name,
                'role' => $user->aclRole->name,
            ];
            $message = trans('admin::auth.log_login_known', $replace);
        } else {
            $message = trans('admin::auth.log_login_unknown');
        }

        logger()->info($message);
    }

    public function onFailed(Failed $event)
    {
        /** @var User $user */
        $user = $event->user;
        $credentials = $event->credentials;

        $replace = [
            'credentials' => implode(':', $credentials),
        ];
        if ($user && ($user instanceof User)) {
            $replace += [
                'name' => $user->name,
                'role' => $user->aclRole->name,
            ];
            $message = trans('admin::auth.log_failed_known', $replace);
        } else {
            $message = trans('admin::auth.log_failed_unknown', $replace);
        }

        logger()->warning($message);
    }

    public function onLockout(Lockout $event)
    {
        $request = $event->request;
        $ip = $request->ip();

        $message = trans('admin::auth.log_lockout', compact('ip'));
        logger()->warning($message);
    }

    public function onPasswordChanged(User $user)
    {
        if ($user->isDirty('password')) {
            $replace = [
                'name' => $user->name,
                'email' => $user->email,
                'role' => $user->aclRole->name,
            ];
            $message = trans('admin::auth.log_password', $replace);
            logger()->warning($message);
        }
    }
}
