<?php

namespace AltSolution\Admin\Locale;

use Illuminate\Support\Arr;

/**
 * Use on models to add multilangual support
 */
trait TranslateTrait
{
    /**
     * Override to add support for multilingual
     * @return array
     */
    public function getFillable()
    {
        $fillable = [];
        foreach (parent::getFillable() as $field) {
            if (strpos($field, '*') === false) {
                $fillable[] = $field;
            } else {
                foreach (cms_locales() as $locale) {
                    $fillable[] = str_replace('*', $locale, $field);
                }
            }
        }

        return $fillable;
    }

    /**
     * Translate model property
     * @param string $property
     * @param string $locale
     * @return string
     */
    public function trans($property, $locale = null)
    {
        if ($locale === null) {
            $locale = app('config')->get('app.locale');
        }

        $value = Arr::get($this, $property . '_' . $locale);
        if (empty($value)) {
            $fallbackLocale = app('config')->get('app.fallback_locale');
            if ($fallbackLocale !== $locale) {
                $value = Arr::get($this, $property . '_' . $fallbackLocale);
            }
        }

        return $value;
    }

}
