<?php

namespace AltSolution\Admin\Models;

use AltSolution\Admin\Locale\TranslateInterface;
use AltSolution\Admin\Locale\TranslateTrait;
use AltSolution\Admin\Modules\Menu\MenuItemModelInterface;
use Illuminate\Database\Eloquent\Model;

/**
 * @property int $id
 * @property int $menu_id
 * @property Menu $menu
 * @property int $parent_id
 * @property MenuItem $parent
 * @property string $url
 * @property bool $target
 */
class MenuItem extends Model implements TranslateInterface, MenuItemModelInterface
{
    use TranslateTrait;

	protected $table = 'menus_items';
    public $timestamps = false;

    protected $fillable = [
        'menu_id',
        'parent_id',
        'name_*',
        'url',
        'target'
    ];

    public function scopeParent($query, $id = 0)
    {
        return $query->where('parent_id', '=', '0')->where('id', '!=', $id);
    }

    public function menu()
    {
        return $this->belongsTo(Menu::class);
    }

    public function child()
    {
        return $this->hasMany($this, 'parent_id')->orderBy('sort');
    }

    public function parent()
    {
        return $this->belongsTo($this, 'parent_id');
    }
}
