<?php

namespace AltSolution\Admin\Models;

use AltSolution\Admin\Image\ImagesInterface;
use AltSolution\Admin\Image\ImagesTrait;
use AltSolution\Admin\Modules\User\UserModelInterface;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;

/**
 * @property int $id
 * @property string $name
 * @property string $email
 * @property int $acl_role_id
 * @property AclRole $aclRole
 * @property bool $is_active
 * @property string $avatar
 * @property string $password
 * @property string $remember_token
 */
class User extends Authenticatable implements ImagesInterface, UserModelInterface
{
    use HasFactory, Notifiable;
    use ImagesTrait;

    public function getImagesFields()
    {
        return [
            'avatar' => [
                'list' => ['crop', 200, 200],
            ],
        ];
    }

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'email',
        // 'password',
        'acl_role_id',
        'is_active',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    public function aclRole()
    {
        return $this->belongsTo(AclRole::class);
    }
}
