<?php

namespace AltSolution\Admin\Seo;

use AltSolution\Admin\Events\SeoPageUpdated;
use AltSolution\Admin\Models\SeoPage;

class SeoRepository implements SeoRepositoryInterface
{
    /**
     * @param string|null $page
     * @return SeoPage[]
     */
    private function pageProperties($page = null)
    {
        $q = SeoPage::query();
        if ($page !== null) {
            $q->where('page', $page);
        }
        $properties = $q->get();

        return $properties;
    }

    public function getPageProperties($name)
    {
        $cacheKey = 'seo_page_' . $name;

        return cache()->remember($cacheKey, config('seo.cache_ttl'), function () use ($name) {
            $properties = $this->pageProperties($name);

            $data = [];
            foreach ($properties as $prop) {
                $data[$prop->locale][$prop->key] = $prop->value;
            }

            return $data;
        });
    }

    public function getPagesProperties()
    {
        $properties = $this->pageProperties();
        $data = [];
        foreach ($properties as $prop) {
            $data[$prop->page][$prop->locale][$prop->key] = $prop->value;
        }

        return $data;
    }

    public function setPageProperties($name, array $properties)
    {
        $oldProperties = $this->pageProperties($name);

        $changed = false;

        $updatedIds = [];
        foreach ($properties as $locale => $props) {
            foreach ($props as $propKey => $propValue) {

                if (empty($propValue)) {
                    // remove empty props
                    continue;
                }

                /** @var SeoPage $newProp */
                $newProp = SeoPage::query()->firstOrNew([
                    'page' => $name,
                    'locale' => $locale,
                    'key' => $propKey,
                ]);
                $newProp->value = $propValue;
                if (!$newProp->exists || $newProp->isDirty('value')) {
                    $changed = true;
                }
                $newProp->save();

                $updatedIds[] = $newProp->getKey();
            }
        }

        foreach ($oldProperties as $property) {
            if (in_array($property->getKey(), $updatedIds)) {
                continue;
            }
            $property->delete();
            $changed = true;
        }

        if ($changed) {
            $cacheKey = 'seo_page_' . $name;
            cache()->forget($cacheKey);
            event(new SeoPageUpdated($name));
        }
    }

    public function setPagesProperties(array $properties)
    {
        $oldProperties = $this->pageProperties();

        $updatedIds = [];
        foreach ($properties as $page => $propsByLocale) {

            $changed = false;
            foreach ($propsByLocale as $locale => $props) {
                foreach ($props as $propKey => $propValue) {

                    if (empty($propValue)) {
                        // remove empty props
                        continue;
                    }

                    /** @var SeoPage $newProp */
                    $newProp = SeoPage::query()->firstOrNew([
                        'page' => $page,
                        'locale' => $locale,
                        'key' => $propKey,
                    ]);
                    $newProp->value = $propValue;
                    if (!$newProp->exists || $newProp->isDirty('value')) {
                        $changed = true;
                    }
                    $newProp->save();

                    $updatedIds[] = $newProp->getKey();
                }
            }

            if ($changed) {
                $cacheKey = 'seo_page_' . $page;
                cache()->forget($cacheKey);
                event(new SeoPageUpdated($page));
            }
        }

        foreach ($oldProperties as $property) {
            if (in_array($property->getKey(), $updatedIds)) {
                continue;
            }
            $property->delete();
        }
    }

}
