<?php

namespace AltSolution\Admin\System;

class Layout extends BaseLayout
{
    /**
     * @var User
     */
    private $user;
    /**
     * @var Menu
     */
    private $menu;
    /**
     * @var Breadcrumbs
     */
    private $breadcrumbs;

    public function __construct()
    {
        parent::__construct();
        $this->user = new User;
        $this->menu = new Menu;
        $this->breadcrumbs = new Breadcrumbs;
    }

    public function toArray()
    {
        $baseArr = parent::toArray();
        return array_merge($baseArr, [
            'user' => $this->user,
            'menu' => $this->menu,
            'breadcrumbs' => $this->breadcrumbs,
        ]);
    }

    /**
     * @return User
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * @return Menu
     */
    public function getMenu()
    {
        return $this->menu;
    }

    /**
     * @return Breadcrumbs
     */
    public function getBreadcrumbs()
    {
        return $this->breadcrumbs;
    }

    /**
     * Set current menu section
     * @param string $alias
     * @return $this
     * @throws \Exception
     */
    public function setMenuSection($alias)
    {
        $section = $this->menu->getSection($alias);
        $section->setActive(true);

        return $this;
    }

    /**
     * @param string $description
     * @param string|null $url
     * @return $this
     */
    public function addBreadcrumb($description, $url = null)
    {
        $item = new BreadcrumbsItem($description, $url);
        $this->breadcrumbs->addItem($item);

        return $this;
    }

    /**
     * @return $this
     */
    public function hideBreadcrumbs()
    {
        $this->breadcrumbs->setVisible(false);

        return $this;
    }
}